//////////////////////////////////////////////////////////////////////////////////////////
// Project description
// 
// Name: Touch Tracer
// Purpose: General test application for detected blobs
//
// Author: Laurence Muller (info@falcon4ever.com)
// Site: www.lauw.biz
// Date: april 2007
// Licensetype: coming soon ;)
//
// Changes:
// v0.2 - Added trace lines, blob colours and general code cleanup (linux compatibility).
// v0.1 - Initial release.
//
// Todo:
// - Got suggestions?
//
//////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////
// Include files
// 
#include "main.h"

// Used namespaces
using namespace std;

ITouchScreen *screen;
Touch_Tracer ttracer;
const int gridsize = 80;
const int gridsize_s = 8;

int screen_width, screen_height, bitdepth, refreshrate;
bool display_grid = true;
bool smooth_grid = false;

//////////////////////////////////////////////////////////////////////////////////////////
// Main
// 
int main(int argc, char **argv) 
{
	char displaymode[24];
	
	printf("Touch tracer v"VERSION"\n\n"); 

	// Check if command arguments are specified
	if (argc < 5)
	{			
		printf("Error:You must specify the display settings (width height bitdepth refreshrate)\n");			
		#if defined(WIN32)
		printf("example: touch_tracer.exe 1024 768 32 60\n\n");
		#else
		printf("example: ./touch_tracer 1024 768 32 60\n\n");
		#endif
		return 0;
	}

	// Setup touchlib
	screen = TouchScreenDevice::getTouchScreen();

    //screen->setDebugMode(false);
	if(!screen->loadConfig("config.xml"))
	{
		screen->pushFilter("cvcapture", "capture1");
		screen->pushFilter("mono", "mono2");
		screen->pushFilter("smooth", "smooth3");
		screen->pushFilter("backgroundremove", "background4");
		screen->pushFilter("brightnesscontrast", "bc5");
		screen->pushFilter("rectify", "rectify6");
		screen->setParameter("rectify6", "level", "25");
		screen->setParameter("capture1", "source", "cam");
		//screen->setParameter("capture1", "source", "hard-5point.avi");
		screen->setParameter("bc5", "brightness", "0.1");
		screen->setParameter("bc5", "contrast", "0.4");
		screen->saveConfig("config.xml");
	}
	screen->registerListener((ITouchListener *)&ttracer);
	
	// Note: Begin processing should only be called after the screen is set up
	screen->beginProcessing();
	screen->beginTracking();

	// Setup glut
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA);
	
	// Convert parsed values
	screen_width = (int)strtol(argv[1], NULL, 0);
	screen_height = (int)strtol(argv[2], NULL, 0);
	bitdepth = (int)strtol(argv[3], NULL, 0);
	refreshrate = (int)strtol(argv[4], NULL, 0);
	
	// Set display mode
	sprintf(displaymode, "%dx%d:%d@%d", screen_width, screen_height, bitdepth, refreshrate);
	glutGameModeString(displaymode);
	
	if (glutGameModeGet(GLUT_GAME_MODE_POSSIBLE)) 
	{
		glutEnterGameMode();
		ttracer.set_screenproperties(screen_width, screen_height, bitdepth, refreshrate);		
	}
	else 
	{
		printf("The select mode is not available\n");
		exit(1);	
	}

	opengl_init();

	glutDisplayFunc(render_scene);
	glutIdleFunc(render_scene);
	glutKeyboardFunc(keyboard_input);
	glutMainLoop();	// Endless loop

	TouchScreenDevice::destroy();

	return 0;
}

// Init opengl defaults
void opengl_init()
{
	glClearColor(0.0f, 0.0f, 0.5f, 0.0f);		// Blue Background	
	glClearDepth(1.0f);							// Depth Buffer Setup	

	glEnable(GL_BLEND); 
	glDisable(GL_DEPTH_TEST);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE);			// Blending Function For Translucency Based On Source Alpha Value
}

// Handle keyboard input
void keyboard_input(unsigned char key, int x, int y)
{
	switch (key) 
	{
		case 'g':
			display_grid = !display_grid;
			break;	
		case 's':
			smooth_grid = !smooth_grid;

			if(smooth_grid == true)
			{
				glEnable(GL_LINE_SMOOTH);					// Nice lines :) (horizontal grid lines won't be drawn in linux)
				glHint(GL_LINE_SMOOTH_HINT, GL_DONT_CARE);
			}
			else
			{
				glDisable(GL_LINE_SMOOTH);
			}
			break;	
		case 'b':			
			/*
				screen->setParameter("background4", "capture", ""); 				
				ttracer.reset_blobs();		
			*/
			break;	
		case 27:  /*  Escape Key */
			screen->saveConfig("config.xml");
			exit(0);
			break;
		default:
			break;
	}
}

void render_scene()
{
	// Processing?
	process_blobs();

	// Clear The Screen And The Depth Buffer
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	// Set view to Ortho2D for easy 2D positioning
	glViewport(0, 0, screen_width, screen_height);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluOrtho2D(0, screen_width, screen_height, 0);
	glMatrixMode(GL_MODELVIEW);

	// Render grid (background)
	if(display_grid)
		render_grid();

	// Draw scene
	ttracer.draw_blobs();	

	// Swap buffers!
	glFlush();
	glutSwapBuffers();
}

void process_blobs()
{
	screen->getEvents();	
}

/*
	Note:
	coordinate system is like this (screen of 1024 x 768)
	[Windows]
	Range x = 0 ... 1023
	Range y = 1 ... 768
   0,1   1023,1
	+---+
	|   |
	+---+
 0,768   1023,768
*/
void render_grid()
{	
	#if defined(WIN32)
	int x_first = 1;
	#else
	int x_first = 0;
	#endif

	glBegin(GL_LINES);
		// small blocks
		glColor3f(0.0f, 0.0f, 0.2f);

		for(int x=x_first; x <= screen_width; x += gridsize_s)
		{
			glVertex2f(x, 1);
			glVertex2f(x, screen_height);			
		}
		for(int y=1; y <= screen_height; y += gridsize_s)
		{
			glVertex2f(0, y);
			glVertex2f(screen_width, y);			
		}

		// large blocks
		glColor3f(0.0f, 0.0f, 0.5f);
		for(int x=x_first; x <= screen_width; x += gridsize)
		{
			glVertex2f(x, 1);
			glVertex2f(x, screen_height);			
		}
		for(int y=1; y <= screen_height; y += gridsize)
		{
			glVertex2f(0, y);
			glVertex2f(screen_width, y);			
		}
	glEnd();		
}

